#include <AH/CUtil/SecureArray.h>
#include <AH/Test/UnitTest.h>

using namespace AH;

//-----------------
// our test class:
//-----------------
class Test: public UnitTest
{
    // test functions, to be called in Test::run():
    void test_getNumEntries();
    void test_setVal();
    void test_copy();
    void test_copyAll();

    // run all tests:
    virtual void run(const std::vector<std::string>&)
    {
	this->test_getNumEntries();
	this->test_setVal();
	this->test_copy();
	this->test_copyAll();
    }

 public:
    // constructor
    Test(): UnitTest("class CUtilSecureArray", std::cout) {}

}; 

UnitTest* theUnitTest =  new Test();


void Test::test_getNumEntries()
{ 
    this->announceTestSeries("getNumEntries()");

    // run the tests:
    UNIT_TEST_BEGIN
    const unsigned int C_SIZE = 20;
    char c_array[C_SIZE];
    this->checkEqual("c_array[20]", 
		     CUtilSecureArray::getNumEntries(c_array), C_SIZE);
    UNIT_TEST_END


    UNIT_TEST_BEGIN
    const unsigned int I_SIZE = 15;
    int i_array[I_SIZE];
    this->checkEqual("i_array[15]", 
		     CUtilSecureArray::getNumEntries(i_array), I_SIZE);
    UNIT_TEST_END


    UNIT_TEST_BEGIN
    const unsigned int D_SIZE = 4;
    double d_array[D_SIZE];
    this->checkEqual("d_array[4]", 
		     CUtilSecureArray::getNumEntries(d_array), D_SIZE);
    UNIT_TEST_END

} // end of Test::test_getNumEntries()



void Test::test_setVal()
{ 
    this->announceTestSeries("setVal");

    // run the tests:
    UNIT_TEST_BEGIN

    const unsigned int C_SIZE = 5;
    char c_array[C_SIZE];
    for (unsigned int i = 0; i < C_SIZE; ++i)
	c_array[i] = 'x';

    CUtilSecureArray::setVal(c_array, 'a');
    for (unsigned int i = 0; i < C_SIZE; ++i)
	this->checkEqual("c_array[" + this->asString(i) + "]",
			 c_array[i], 'a'); 

    UNIT_TEST_END


    UNIT_TEST_BEGIN

    const unsigned int I_SIZE = 6;
    int i_array[I_SIZE];
    for (unsigned int i = 0; i < I_SIZE; ++i)
	i_array[i] = -1;
    CUtilSecureArray::setVal(i_array, +5);
    for (unsigned int i = 0; i < I_SIZE; ++i)
	this->checkEqual("i_array[" + this->asString(i) + "]",
			 i_array[i], +5); 

    UNIT_TEST_END


    UNIT_TEST_BEGIN

    const unsigned int D_SIZE = 4;
    double d_array[D_SIZE];
    for (unsigned int i = 0; i < D_SIZE; ++i)
	d_array[i] = 2.71 * i;
    CUtilSecureArray::setVal(d_array, 1.4142);
    for (unsigned int i = 0; i < D_SIZE; ++i)
	this->checkEqual("d_array[" + this->asString(i) + "]",
			 d_array[i], 1.4142, 0.00001); 

    UNIT_TEST_END

} // end of Test::test_setVal()



void Test::test_copy()
{ 
    this->announceTestSeries("copy()");

    UNIT_TEST_BEGIN

    const unsigned int C_SIZE = 5;
    char c_src[C_SIZE];
    char c_dest[C_SIZE];
    for (unsigned int i = 0; i < C_SIZE; ++i)
    {
	c_src[i] = 's';
	c_dest[i] = 'd';
    }

    int retval = CUtilSecureArray::copy(c_dest, c_src);
    this->checkEqual("return value", retval, 0); 

    for (unsigned int i = 0; i < C_SIZE; ++i)
	this->checkEqual("c_dest[" + this->asString(i) + "]",
			 c_dest[i], 's'); 

    UNIT_TEST_END


    UNIT_TEST_BEGIN

    const unsigned int IS_SIZE = 4;
    const unsigned int ID_SIZE = 6;
    int i_src[IS_SIZE];
    int i_dest[ID_SIZE];
    for (unsigned int i = 0; i < IS_SIZE; ++i)
	i_src[i] = 5;
    for (unsigned int i = 0; i < ID_SIZE; ++i)
	i_dest[i] = -1;

    int retval = CUtilSecureArray::copy(i_dest, i_src);
    this->checkEqual("return value", retval, (int)IS_SIZE - (int)ID_SIZE); 

    for (unsigned int i = 0; i < IS_SIZE; ++i)
	this->checkEqual("i_dest[" + this->asString(i) + "]",
			 i_dest[i], 5); 
    for (unsigned int i = IS_SIZE; i < ID_SIZE; ++i)
	this->checkEqual("i_dest[" + this->asString(i) + "]",
			 i_dest[i], -1); 

    UNIT_TEST_END

} // end of Test::test_copy()

void Test::test_copyAll()
{ 
    this->announceTestSeries("copyAll");

    UNIT_TEST_BEGIN

    const unsigned int CS_SIZE = 4;
    const unsigned int CD_SIZE = 6;
    char c_src[CS_SIZE];
    char c_dest[CD_SIZE];
    for (unsigned int i = 0; i < CS_SIZE; ++i)
	c_src[i] = 's';
    for (unsigned int i = 0; i < CD_SIZE; ++i)
	c_dest[i] = 'd';
    const char DEFVAL = 'f';

    int retval = CUtilSecureArray::copyAll(c_dest, c_src, DEFVAL);
    this->checkEqual("return value", retval, (int)CS_SIZE - (int)CD_SIZE); 

    for (unsigned int i = 0; i < CS_SIZE; ++i)
	this->checkEqual("c_dest[" + this->asString(i) + "]",
			 c_dest[i], 's'); 
    for (unsigned int i = CS_SIZE; i < CD_SIZE; ++i)
	this->checkEqual("c_dest[" + this->asString(i) + "]",
			 c_dest[i], DEFVAL); 

    UNIT_TEST_END


    UNIT_TEST_BEGIN

    const unsigned int IS_SIZE = 6;
    const unsigned int ID_SIZE = 4;
    int i_src[IS_SIZE];
    int i_dest[ID_SIZE];
    for (unsigned int i = 0; i < IS_SIZE; ++i)
	i_src[i] = 5;
    for (unsigned int i = 0; i < ID_SIZE; ++i)
	i_dest[i] = -1;
    const int DEFVAL = 3;

    int retval = CUtilSecureArray::copyAll(i_dest, i_src, DEFVAL);
    this->checkEqual("return value", retval, (int)IS_SIZE - (int)ID_SIZE); 

    for (unsigned int i = 0; i < ID_SIZE; ++i)
	this->checkEqual("i_dest[" + this->asString(i) + "]",
			 i_dest[i], 5); 

    UNIT_TEST_END


    UNIT_TEST_BEGIN

    const unsigned int DS_SIZE = 5;
    const unsigned int DD_SIZE = 5;
    double d_src[DS_SIZE];
    double d_dest[DD_SIZE];
    for (unsigned int i = 0; i < DS_SIZE; ++i)
	d_src[i] = 1.4142;
    for (unsigned int i = 0; i < DD_SIZE; ++i)
	d_dest[i] = 3.14 * i;
    const double DEFVAL = 2.71;

    int retval = CUtilSecureArray::copyAll(d_dest, d_src, DEFVAL);
    this->checkEqual("return value", retval, (int)DS_SIZE - (int)DD_SIZE); 

    for (unsigned int i = 0; i < DD_SIZE; ++i)
	this->checkEqual("d_dest[" + this->asString(i) + "]",
			 d_dest[i], 1.4142); 

    UNIT_TEST_END
} // end of Test::test_copyAll()

